// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.
'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable:no-any max-func-body-length
if (Reflect.metadata === undefined) {
    // tslint:disable-next-line:no-require-imports no-var-requires
    require('reflect-metadata');
}
const path = require("path");
const stream_1 = require("stream");
const vscode_debugadapter_1 = require("vscode-debugadapter");
const logger_1 = require("vscode-debugadapter/lib/logger");
const messages_1 = require("vscode-debugadapter/lib/messages");
const helpers_1 = require("../common/helpers");
const DebugFactory_1 = require("./DebugClients/DebugFactory");
const serviceRegistry_1 = require("./serviceRegistry");
const types_1 = require("./types");
const DEBUGGER_CONNECT_TIMEOUT = 10000;
const MIN_DEBUGGER_CONNECT_TIMEOUT = DEBUGGER_CONNECT_TIMEOUT / 2;
class PythonDebugger extends vscode_debugadapter_1.DebugSession {
    constructor(serviceContainer, isServer) {
        super(false, isServer);
        this.serviceContainer = serviceContainer;
        this.client = helpers_1.createDeferred();
    }
    static run() {
        return __awaiter(this, void 0, void 0, function* () {
            const serviceContainer = serviceRegistry_1.initializeIoc();
            const debugStreamProvider = serviceContainer.get(types_1.IDebugStreamProvider);
            const { input, output } = yield debugStreamProvider.getInputAndOutputStreams();
            const isServerMode = debugStreamProvider.useDebugSocketStream;
            const protocolMessageWriter = serviceContainer.get(types_1.IProtocolMessageWriter);
            // tslint:disable-next-line:no-empty
            vscode_debugadapter_1.logger.init(() => { }, path.join(__dirname, '..', '..', '..', 'experimental_debug.log'));
            const stdin = input;
            const stdout = output;
            try {
                stdin.pause();
                const handshakeDebugOutStream = new stream_1.PassThrough();
                const handshakeDebugInStream = new stream_1.PassThrough();
                const throughOutStream = new stream_1.PassThrough();
                const throughInStream = new stream_1.PassThrough();
                const inputProtocolParser = serviceContainer.get(types_1.IProtocolParser);
                inputProtocolParser.connect(throughInStream);
                const outputProtocolParser = serviceContainer.get(types_1.IProtocolParser);
                outputProtocolParser.connect(throughOutStream);
                const protocolLogger = serviceContainer.get(types_1.IProtocolLogger);
                protocolLogger.connect(throughInStream, throughOutStream);
                // Keep track of the initialize message, we'll need to re-send this to ptvsd, for bootstrapping.
                const initializeRequest = new Promise(resolve => {
                    inputProtocolParser.on('request_initialize', (data) => {
                        resolve(data);
                        inputProtocolParser.dispose();
                    });
                });
                throughOutStream.pipe(stdout);
                handshakeDebugOutStream.pipe(throughOutStream);
                // Lets start our debugger.
                const session = new PythonDebugger(serviceContainer, isServerMode);
                session.setRunAsServer(isServerMode);
                let terminatedEventSent = false;
                function dispose() {
                    if (!terminatedEventSent) {
                        protocolMessageWriter.write(stdout, new vscode_debugadapter_1.TerminatedEvent());
                        terminatedEventSent = true;
                    }
                    session.shutdown();
                }
                outputProtocolParser.once('event_terminated', () => {
                    terminatedEventSent = true;
                    dispose();
                });
                if (!isServerMode) {
                    process.on('SIGTERM', dispose);
                }
                session.on('_py_enable_protocol_logging', enabled => {
                    if (enabled) {
                        vscode_debugadapter_1.logger.setup(logger_1.LogLevel.Verbose, true);
                        protocolLogger.setup(vscode_debugadapter_1.logger);
                    }
                    else {
                        protocolLogger.dispose();
                    }
                });
                outputProtocolParser.on('response_launch', () => __awaiter(this, void 0, void 0, function* () {
                    const debuggerSocket = yield session.debugServer.client;
                    debuggerSocket.on('end', dispose);
                    debuggerSocket.on('error', dispose);
                    const debugSoketProtocolParser = serviceContainer.get(types_1.IProtocolParser);
                    debugSoketProtocolParser.connect(debuggerSocket);
                    // The PTVSD process has launched, now send the initialize request to it.
                    const request = yield initializeRequest;
                    protocolMessageWriter.write(debuggerSocket, request);
                    // Wait for PTVSD to reply back with initialized event.
                    debugSoketProtocolParser.once('event_initialized', (initialized) => {
                        throughInStream.unpipe(handshakeDebugInStream);
                        throughInStream.pipe(debuggerSocket);
                        debuggerSocket.pipe(throughOutStream);
                        // Forward the initialized event sent by PTVSD onto VSCode.
                        protocolMessageWriter.write(throughOutStream, initialized);
                    });
                }));
                throughInStream.pipe(handshakeDebugInStream);
                stdin.pipe(throughInStream);
                session.start(handshakeDebugInStream, handshakeDebugOutStream);
                stdin.resume();
            }
            catch (ex) {
                vscode_debugadapter_1.logger.error(`Debugger crashed.${ex.message}`);
                protocolMessageWriter.write(stdout, new messages_1.Event('error', `Debugger Error: ${ex.message}`));
                protocolMessageWriter.write(stdout, new vscode_debugadapter_1.OutputEvent(ex.toString(), 'stderr'));
            }
        });
    }
    shutdown() {
        if (this.debugServer) {
            this.debugServer.Stop();
            this.debugServer = undefined;
        }
        if (this.debugClient) {
            this.debugClient.Stop();
            this.debugClient = undefined;
        }
        super.shutdown();
    }
    initializeRequest(response, args) {
        const body = response.body;
        body.supportsExceptionInfoRequest = true;
        body.supportsConfigurationDoneRequest = true;
        body.supportsConditionalBreakpoints = true;
        body.supportsSetVariable = true;
        body.supportsExceptionOptions = true;
        body.exceptionBreakpointFilters = [
            {
                filter: 'raised',
                label: 'Raised Exceptions',
                default: true
            },
            {
                filter: 'uncaught',
                label: 'Uncaught Exceptions',
                default: true
            }
        ];
        if (typeof args.supportsRunInTerminalRequest === 'boolean') {
            this.supportsRunInTerminalRequest = args.supportsRunInTerminalRequest;
        }
        this.sendResponse(response);
    }
    attachRequest(response, args) {
        this.sendResponse(response);
    }
    launchRequest(response, args) {
        const enableLogging = args.logToFile === true;
        this.emit('_py_enable_protocol_logging', enableLogging);
        this.startPTVSDDebugger(args)
            .then(() => this.waitForDebuggerConnection(args))
            .then(() => this.sendResponse(response))
            .catch(ex => {
            const message = this.getErrorUserFriendlyMessage(args, ex) || 'Debug Error';
            this.sendErrorResponse(response, { format: message, id: 1 }, undefined, undefined, vscode_debugadapter_1.ErrorDestination.User);
        });
    }
    startPTVSDDebugger(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const launcher = DebugFactory_1.CreateLaunchDebugClient(args, this, this.supportsRunInTerminalRequest);
            this.debugServer = launcher.CreateDebugServer(undefined, this.serviceContainer);
            const serverInfo = yield this.debugServer.Start();
            return launcher.LaunchApplicationToDebug(serverInfo);
        });
    }
    waitForDebuggerConnection(args) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                let rejected = false;
                const duration = this.getConnectionTimeout(args);
                const timeout = setTimeout(() => {
                    rejected = true;
                    reject(new Error('Timeout waiting for debugger connection'));
                }, duration);
                try {
                    yield this.debugServer.client;
                    timeout.unref();
                    if (!rejected) {
                        resolve();
                    }
                }
                catch (ex) {
                    reject(ex);
                }
            }));
        });
    }
    getConnectionTimeout(args) {
        const connectionTimeout = typeof args.connectionTimeout === 'number' ? args.connectionTimeout : DEBUGGER_CONNECT_TIMEOUT;
        return Math.max(connectionTimeout, MIN_DEBUGGER_CONNECT_TIMEOUT);
    }
    getErrorUserFriendlyMessage(launchArgs, error) {
        if (!error) {
            return;
        }
        const errorMsg = typeof error === 'string' ? error : ((error.message && error.message.length > 0) ? error.message : '');
        if (helpers_1.isNotInstalledError(error)) {
            return `Failed to launch the Python Process, please validate the path '${launchArgs.pythonPath}'`;
        }
        else {
            return errorMsg;
        }
    }
}
exports.PythonDebugger = PythonDebugger;
PythonDebugger.run().catch(ex => {
    // Not necessary except for perhaps debugging.
});
//# sourceMappingURL=mainV2.js.map